/////////////////////////////////////////////////////////////////////////////
// Name:        ServerData.cpp
// Purpose:     Store the server parameters here
// Author(s):   Atul, Ashesh
// Created:     2008-07-08
// RCS-ID:      $Id: ServerData.cpp,v 1.8 2008/08/19 13:24:56 asheshv Exp $
// Copyright:   (c) EnterpriseDB
/////////////////////////////////////////////////////////////////////////////

#include "Server.h"
#include "WSUtils.h"

#ifdef WIN32
#define STRICMP(str1, str2) _stricmp(str1, str2)
#else
#define STRICMP(str1, str2) strcasecmp(str1, str2)
#endif //WIN32

const wxString ServerConfigData::ms_parameters[DYNATUNE_CONFIG_LEN] =
{
    wxT("max_connections"),
    wxT("shared_buffers"),
    wxT("work_mem"),
    wxT("maintenance_work_mem"),
    wxT("max_fsm_pages"),
    wxT("max_fsm_relations"),
    wxT("wal_buffers"),
    wxT("checkpoint_segments"),
    wxT("random_page_cost"),
    wxT("effective_cache_size"),
    wxT("autovacuum"),
    wxT("autovacuum_naptime"),
    wxT("autovacuum_vacuum_threshold"),
    wxT("autovacuum_analyze_threshold"),
    wxT("autovacuum_vacuum_scale_factor"),
    wxT("autovacuum_analyze_scale_factor"),
    wxT("stats_row_level")
};

bool ServerConfigData::ms_ConfigFlags[DYNATUNE_CONFIG_LEN] = {false};

/**
*  Name: ClearConfigFlags
*  Desc: Clear the configuration flags
*        - Use this function, before making first time call to 
*          UpdateConfigBuffer, while updating the postgresql.conf
*          (configuration file).
*        - It will be useful to maintain the list of updated
*          parameters for that particular configuration files.
*          
*/
void Server::ClearConfigFlags()
{
    for(int i =0; i < DYNATUNE_CONFIG_LEN; i++)
    {
        ServerConfigData::ms_ConfigFlags[i] = false;
    }
}

/**
*  Name: UpdateConfigBuffer
*  Desc: Update the buffer, if it matches following criteria:
*        1. Format of the buffer matches to "parameter = value #some_comment"
*        2. buffer contains one of the parameter from the parameter list 
*           (ms_parameters)
*        3. The configuration has valid value for that particular parameter
*        Resulted buffer will in the following format:
*           "parameter = new_value #some_comment"
*/
wxString Server::UpdateConfigBuffer(wxString buffer, 
                                    wxString& matched_parameter,
                                    wxString& original_value,
                                    wxString& new_value)
{
    int matched_parameter_index = -1;
    int parse_index = -1;
    wxString comments;

    // Initialize the matched_parameter to Empty
    matched_parameter.Empty();
    // Initialize the original value to Empty
    original_value.Empty();

    /**
    * Itreate through all the parameter list, to check if 
    * this buffer contains one of the parameter in the following
    * format:
    *    parameter = value
    * We can have zero or more no of whitespace in between this
    * Keywords
    */
    for(int i=0; i < DYNATUNE_CONFIG_LEN; i++)
    {
        size_t index = (unsigned int)-1;
        if((index = buffer.find(ServerConfigData::ms_parameters[i])) != -1)
        {
            int parameterLen = ServerConfigData::ms_parameters[i].Len();
            if(index != 0)
            {
                char tmpChar = buffer.at(index - 1);
                if(tmpChar != ' ' && tmpChar != '\t')
                    continue;
            }
            while(buffer.at(index + parameterLen) == ' ' ||
                buffer.at(index + parameterLen) == '\t')
                index++;
            if(buffer.at(index + parameterLen) == '=') {
                matched_parameter_index = i;
                parse_index = index + parameterLen + 1;
                break;
            }
        }
    }

    // No parameter matched with the current buffer
    if(matched_parameter_index == -1)
        return buffer;

    // Check if this parameter found multiple times
    // If found it again, return an empty string 
    if(ServerConfigData::ms_ConfigFlags[matched_parameter_index])
        return wxT("");

    /**
    *  If value for the matched parameter is empty, then
    *  we will not modify the values it.
    */
    if(m_cfgData.m_parameter_value[matched_parameter_index].IsEmpty())
    {
        return buffer;
    }

    ServerConfigData::ms_ConfigFlags[matched_parameter_index] = true;
    matched_parameter = ServerConfigData::ms_parameters[matched_parameter_index];

    // Look for white-spaces before the original value
    char current_char;
    int buffer_length = (int)buffer.Len();
    while(parse_index < buffer_length && (current_char = (char)buffer.at(parse_index++)) == ' ' ||
        current_char == '\t');

    if(parse_index <= buffer_length)
        original_value = wxString::Format(_("%c"), (char)current_char);
    //Fetching original value
    while(parse_index < buffer_length && (current_char = (char)buffer.at(parse_index++)) != ' ' &&
        current_char != '\t' && current_char != '#')
        original_value += current_char;

    if(parse_index < buffer_length)
        original_value = wxString::Format(_("%c"), (char)current_char);

    // Fetching rest of the string as comment
    if(parse_index < buffer_length)
        comments += buffer.SubString(parse_index, buffer.Len());

    new_value = m_cfgData.m_parameter_value[matched_parameter_index];

    return 
        ServerConfigData::ms_parameters[matched_parameter_index] + wxT(" = ") +
        new_value + comments;
}

/**
*  Name: SetValue
*  Desc: Set the value for the particular parameter, if it is found 
*        in the list of parameter list. (ms_parameters)
*/
void ServerConfigData::SetValue(wxString &prop, wxString &value) {
    for(int i=0; i < DYNATUNE_CONFIG_LEN; i++)
    {
        if(STRICMP(
            (const char *)ms_parameters[i].mb_str(wxConvUTF8),
            (const char *) prop.mb_str(wxConvUTF8)) == 0)
        {
            m_parameter_value[i] = value;
            break;
        }
    }
}

wxString Server::GetNextNonUpdatedConfiguration()
{
    for(int i=0; i < DYNATUNE_CONFIG_LEN; i++)
    {
        if(ServerConfigData::ms_ConfigFlags[i] == false)
        {
            ServerConfigData::ms_ConfigFlags[i] = true;
            if(ServerConfigData::ms_parameters[i] == wxT("stats_row_level"))
                return wxEmptyString;
            return ServerConfigData::ms_parameters[i] +
                wxT(" = ") +
                m_cfgData.m_parameter_value[i];
        }
    }
    return wxEmptyString;
}

