/////////////////////////////////////////////////////////////////////////////
// Name:        SysInfoPage.cpp
// Purpose:     Wizard page for gathering system info (workload profile, 
//				server utilization
// Author(s):   Atul, Ashesh
// Created:     2008-07-08
// RCS-ID:      $Id: SysInfoPage.cpp,v 1.15 2008/08/21 06:43:12 atuld Exp $
// Copyright:   (c) EnterpriseDB
/////////////////////////////////////////////////////////////////////////////

#include "TuningTool.h"
#include "Wizard.h"
#include "WSUtils.h"
#include "WelcomePage.h"
#include "ProxyDialog.h"
#ifdef WIN32
#include "gsoapWinInet.h"
#endif /* WIN32 */

// wxWindows headers
#include <wx/wx.h>
#include <wx/busyinfo.h>
#include <wx/utils.h>
#include <wx/wizard.h>
#include <wx/textfile.h>
#include <wx/gbsizer.h>

#ifdef __WXMSW__
#include <wx/msw/registry.h>
#endif

// Application headers
#include "SysInfoPage.h"

extern const char * Tuning_xpm[];

#define SYSINFO_PROGRESS_RANGE 6
#define TW_USER_ID "tuningwizard@enterprisedb.com"
#define TW_PASSWD  "password"

const int RBX_UTILIZATIONTYPES=1002;
const int RBX_WORKLOADPROFILE=1003;

BEGIN_EVENT_TABLE(SysInfoPage, wxWizardPageSimple)
EVT_WIZARD_PAGE_CHANGING(wxID_ANY, SysInfoPage::OnWizardPageChanging)
EVT_RADIOBOX(RBX_UTILIZATIONTYPES, SysInfoPage::OnUtilizationTypesClicked)
EVT_RADIOBOX(RBX_WORKLOADPROFILE, SysInfoPage::OnWorkloadProfileClicked)
END_EVENT_TABLE()

const wxString SysInfoPage::m_utilizationTypesHelpText[3] = 
{
    wxT("\nThis is a development machine and many other applications "\
        "will be running on it. Stress testing should not be preformed with "\
        "this configuraiton. PostgreSQL will use a minimal amount of memory"),
    wxT("\nSeveral Applications will be running on this machine. Choose this "\
        "option for web/application servers."),
    wxT("\nThis machine is dedicated to run PostgreSQL and will use available "\
        "memory to optimize performance.")
};

const wxString SysInfoPage::m_workloadProfileHelpText[3] =
{
    wxT("\nThe running application is transaction-intensive application."),
    wxT("\nThe database will be used for both transaction processing as well "\
        "as complex queries and reporting."),
    wxT("\nThe database will be used for reporting applications.")
};

SysInfoPage::SysInfoPage(wxWizard *parent)
: wxWizardPageSimple(parent)
{
    SetBackgroundColour(*wxWHITE);
    wxBoxSizer *mainSizer = new wxBoxSizer(wxVERTICAL);

    mainSizer->Add(0, 10);

    wxStaticText *st = new wxStaticText(this, wxID_ANY, 
        wxT("Please select the type of usage required for this machine."));
    st->Wrap(400);
    mainSizer->Add(st, 0, wxALL, 5);

    wxBoxSizer *utilizationSizer = new wxBoxSizer(wxHORIZONTAL);
    static const wxString utilizationTypes[] =
    {
        wxT("Development"),
        wxT("Mixed"),
        wxT("Dedicated")
    };
    m_serverUtilization = new wxRadioBox(
                                this, RBX_UTILIZATIONTYPES,
                                wxT("Server Utilization"), wxDefaultPosition,
                                wxDefaultSize, WXSIZEOF(utilizationTypes),
                                utilizationTypes, 1, wxRA_SPECIFY_COLS);
    utilizationSizer->Add(m_serverUtilization, 0, wxGROW | wxALL, 5);
    m_utilizationTypeDefaultHelpText = new wxStaticText(this, wxID_ANY, m_utilizationTypesHelpText[0]);
    m_utilizationTypeDefaultHelpText->Wrap(275);
    utilizationSizer->Add(m_utilizationTypeDefaultHelpText, 0, wxGROW | wxALL, 5);
    mainSizer->Add(utilizationSizer, 0, wxGROW | wxALL, 5);

    /*
    wxBoxSizer *workloadProfileSizer = new wxBoxSizer(wxHORIZONTAL);
    static const wxString workloadProfiles[] =
    {
        wxT("Transaction processing"),
        wxT("General Purpose"),
        wxT("Reporting")
    };
    m_workloadProfile = new wxRadioBox(this, RBX_WORKLOADPROFILE,
                              wxT("Workload Profile"), wxDefaultPosition,
                              wxDefaultSize, WXSIZEOF(workloadProfiles),
                              workloadProfiles, 1, wxRA_SPECIFY_COLS);
    workloadProfileSizer->Add(m_workloadProfile, 0, wxGROW | wxALL, 5);

    m_workloadProfileDefaultHelpText = new wxStaticText(this, wxID_ANY, m_workloadProfileHelpText[0]);
    m_workloadProfileDefaultHelpText->Wrap(250);
    workloadProfileSizer->Add(m_workloadProfileDefaultHelpText, 0, wxGROW | wxALL, 5);
    mainSizer->Add(workloadProfileSizer, 0, wxGROW | wxALL, 5);
    */

    st = new wxStaticText(this, wxID_ANY, 
        wxT("Upon clicking the 'Next' button, basic details of your hardware "\
            "configuration and PostgreSQL installation will be sent to the "\
            "EnterpriseDB servers which will calculate configuration values "\
            "for your PostgreSQL server."));
    st->Wrap(390);
    mainSizer->Add(st, 0, wxALL, 5);

    st = new wxStaticText(this, wxID_ANY,
        wxT("No data that could identify you or your machine will be sent, "\
            "and no data other than standard webserver logs will be kept "\
            "by EnterpriseDB."));
    mainSizer->Add(st, 0, wxALL | wxALIGN_LEFT, 5);
    st->Wrap(300);

    m_progress = NULL;

    mainSizer->AddStretchSpacer();

    wxGridBagSizer *imageSizer = new wxGridBagSizer();

    imageSizer->SetMinSize(15, 5);
#if wxUSE_STATBMP
    wxBoxSizer *bmpSizer = new wxBoxSizer(wxVERTICAL);
    imageSizer->Add(
        bmpSizer, wxGBPosition(0,32), wxGBSpan(3,1), wxALIGN_RIGHT
        );

    // Create and run the wizard
    wxBitmap bitmap = wxBitmap(Tuning_xpm);

    if ( bitmap.Ok() )
    {
        wxStaticBitmap *statbmp = new wxStaticBitmap(this, wxID_ANY, bitmap);
        bmpSizer->Add(statbmp, 0, wxALL, 0);
    }
#endif
    mainSizer->Add(imageSizer);
    SetSizer(mainSizer);
    mainSizer->Fit(this);
}

void SysInfoPage::OnWizardPageChanging(wxWizardEvent& event)
{
    if(event.GetDirection() == true)
    {
        if(!UpdateServerConfiguration())
            event.Veto();
    }
}

void SysInfoPage::OnUtilizationTypesClicked(wxCommandEvent &event)
{
    m_utilizationTypeDefaultHelpText->SetLabel(m_utilizationTypesHelpText[event.GetSelection()]);
    m_utilizationTypeDefaultHelpText->Wrap(275);
}

void SysInfoPage::OnWorkloadProfileClicked(wxCommandEvent &event)
{
    m_workloadProfileDefaultHelpText->SetLabel(m_workloadProfileHelpText[event.GetSelection()]);
    m_workloadProfileDefaultHelpText->Wrap(250);
}

bool SysInfoPage::UpdateServerConfiguration()
{
    int svr_utilization = GetServerUtillization();
    int workload_profile = GetDBMSUsage();
    Wizard* wizard = (Wizard *)this->GetParent();
    Server* server = ((WelcomePage *)wizard->GetFirstPage())->GetSelectedServer();
    if(server != NULL )
    {
        InitProgressBar(true);
        bool retVal = CallWebservice(server, svr_utilization, workload_profile);
        InitProgressBar(false);
        return retVal;
    }
    ::wxLogError(
        wxT("Unexpected Error!\nPlease visit http://forums.enterprisedb.com"\
            " for further assistance."));
    return false;
}

void SysInfoPage::InitProgressBar(bool show)
{
    if(show)
    {
        if(!m_progress)
        {
            m_progress = new wxProgressDialog(
                wxT("EnterpriseDB's Tuning Wizard"),
                wxT("Initialize Webservice ...            "),
                SYSINFO_PROGRESS_RANGE, this,
                wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_SMOOTH);
            m_progress->Update(0);
        }
    }
    else
    {
        m_progress->Destroy();
        m_progress = NULL;
    }
}

void SysInfoPage::UpdateProgressBar(int value, wxString status)
{
    if(!m_progress)
        return;
    if(value > SYSINFO_PROGRESS_RANGE)
        value = SYSINFO_PROGRESS_RANGE;
    m_progress->Update(value, status);
}

bool SysInfoPage::CallWebservice(Server *server, int svr_utilization, int workload_profile)
{   
    const char webserver[]
#ifndef STAGING_SERVER
        = "https://services.enterprisedb.com/authws/services/"\
          "AuthenticationService?wsdl";
#else
        = "http://services.staging.enterprisedb.com/authws/services/"\
          "AuthenticationService?wsdl";
#endif /* STAGING_SERVER */
    char *strWebService = NULL;
    wxString proxy_host, proxy_port;
    
    AuthenticationServiceSoapBindingProxy authProxy;
#ifdef WIN32
    soap_register_plugin(&authProxy, wininet_plugin );
#endif
    if(server == NULL)
        return false;
    UpdateProgressBar(1,
                wxT("Setting required parameter to connect WebService ..."));
    ProxyDialog::GetProxy(proxy_host, proxy_port);
    if(!proxy_port.IsEmpty())
    {
        long lProxyPort;
        authProxy.proxy_host = proxy_host.mb_str(wxConvUTF8);
        proxy_port.ToLong(&lProxyPort);
        authProxy.proxy_port = (int)lProxyPort;
    }
    authProxy.soap_endpoint = webserver;
    authProxy.userid = TW_USER_ID;
    authProxy.passwd = TW_PASSWD;
    authProxy.password = TW_PASSWD;
    UpdateProgressBar(2, wxT("Initializing WebService ..."));
    
#ifdef linux
    soap_ssl_init(); /* init OpenSSL (just once) */
    /* FIXME: SOAP_SSL_NO_AUTHENTICATION is a hack to overcome the connection
     * problem on Ubuntu */
    if (soap_ssl_client_context(&authProxy,
                                SOAP_SSL_NO_AUTHENTICATION,
                                NULL,
                                NULL,
                                NULL,
                                NULL,
                                NULL
                               ))
    {
        char errorstr[1024];
        authProxy.soap_stream_fault(std::cerr);
        authProxy.soap_sprint_fault(errorstr, 1024);
        UpdateProgressBar(SYSINFO_PROGRESS_RANGE,
                          wxT("Could not connect to WebService ..."));
        ::wxLogError(
                wxT("Error accessing WebService!\nMake sure that you are "\
                    "connected to the network.\nIf you are connected to the "\
                    "internet, please check the proxy settings.\nPlease visit"\
                    " http://forums.enterprisedb.com for further assistance."));
        return false;
    } 
#endif

    if(authProxy.testWebService(strWebService) == SOAP_OK) {
        free(strWebService);
    }
    else if (authProxy.error)
    {
        char errorstr[1024];
        authProxy.soap_stream_fault(std::cerr);
        authProxy.soap_sprint_fault(errorstr, 1024);
        UpdateProgressBar(SYSINFO_PROGRESS_RANGE,
                          wxT("Could not connect to WebService ..."));
        ::wxLogError(
                wxT("Error accessing WebService!\nMake sure that you are "\
                    "connected to the network.\nIf you are connected to the "\
                    "internet, please check the proxy settings.\nPlease visit"\
                    " http://forums.enterprisedb.com for further assistance."));
        return false;
    }
 
    bool isValid = false;

    char *uuid = (char *)WSUtils::Genguid();
    UpdateProgressBar(3, wxT("Checking for WebService ..."));
    wxString wStrDbVer
              = server->m_serverVersion.IsEmpty() ?
                    wxT("8.3") :
                    server->m_serverVersion;
    wxString wStrUUID  = wxString::FromAscii(uuid);
    wxString wStrUser  = wxString::FromAscii(TW_USER_ID);
    wxString wStrPass  = wxString::FromAscii(TW_PASSWD);
    wxString wStrSvrU  = wxString::FromAscii(
                                    g_svr_utilizaton_value[svr_utilization]);
    wxString wStrWP    = wxString::FromAscii(PG_SVR_WORKLOAD_PROF_TP);
    if(
        WSUtils::validateUser(authProxy, wStrUUID, wStrDbVer,
                              wStrUser, wStrPass, wStrSvrU, wStrWP) != 0)
    {
        ::wxLogError(
            wxT("Unexpected Error!\nPlease visit"\
                " http://forums.enterprisedb.com for further assistance."));
        return false;
    }
    int length = 0;
    UpdateProgressBar(4, wxT("Fetching information ..."));
    ResultData *dynatune = WSUtils::GetDynatuneInfo(
        authProxy, wStrUUID, wStrSvrU, wStrWP, length);
    if(dynatune != NULL) {
        UpdateProgressBar(5, wxT("Updating information ..."));
#ifdef ___CREATE_LOG_
        wxTextFile aLog(wxT("edb_tuningwizard.log"));
        aLog.Create();
#endif //___CREATE_LOG_
        for(int index=0; index < length; index++)
        {
            wxString wStrProp = wxString::FromAscii(dynatune[index].prop);
            wxString wStrVal  = wxString::FromAscii(dynatune[index].value);
#ifdef ___CREATE_LOG_
            wxString buffer;
            buffer = wStrProp;
            buffer += wxT("=");
            buffer += wStrVal;
            aLog.AddLine(buffer);
#endif //___CREATE_LOG_
            server->m_cfgData.SetValue(wStrProp, wStrVal);
        }
#ifdef ___CREATE_LOG_
        aLog.Write();
        aLog.Close();
#endif //___CREATE_LOG_
    }
    WSUtils::Releaseguid();
    free(dynatune);
    dynatune = NULL;
    UpdateProgressBar(6, wxT("Done"));
    return true;
}
